﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Text;

/*
 *         Progetto: Calcolatrice online (socket TCP)
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 23/10/2015
 *  
 *      Descrizione: Modulo client - Invia al server un'espressione matematica  
 *                   da calcolare utilizzando un socket TCP.
 *                   Aggiunta la gestione degli eventi di chiusura
 */

namespace ClientSocket
{
    class Program
    {
        const string nomeServerDefault = "localhost";
        const int portaServer = 4000;               // TCP

        static bool forzaChiusura = false;

        static void Main(string[] args)
        {
            SetConsoleCtrlHandler(new HandlerRoutine(ControllaEventiChiusura), true);

            string espressione;
            byte[] bufferRis = new byte[1024];  // Buffer per memorizzare il risultato

            // Il nome del server è ricavato dal primo argomento della linea di comando.
            // In assenza di argomenti si considera il nome di default.
            string nomeServer = args.Length > 0 ? args[0] : nomeServerDefault;

            try
            {
                IPAddress ipServer;
                
                if (!IPAddress.TryParse(nomeServer, out ipServer))
                    // Se il nome del server non è un indirizzo IP occorre risolverlo mediante DNS.
                    ipServer = Array.Find(
                        Dns.GetHostEntry(nomeServer).AddressList,            // Cerca il primo indirizzo IPv4 
                        e => e.AddressFamily == AddressFamily.InterNetwork   // associato al nome del server
                    );    

                Socket client = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

                // Crea una nuova connessione al server
                client.Connect(new IPEndPoint(ipServer, portaServer));

                Console.WriteLine("Connesso a {0}", client.RemoteEndPoint);
                Console.WriteLine();
                Console.WriteLine("Inserire l'espressione matematica da inviare al server.");
                Console.WriteLine("Inserire una riga vuota per terminare.");
                Console.WriteLine();

                do
                {
                    Console.Write(">");

                    espressione = Console.ReadLine();

                    // Se l'utente preme CTRL-C o chiude la finestra, l'espressione è pari a null.
                    if (espressione == null)
                    {
                        Console.WriteLine();
                        espressione = "";
                    }
                    else
                        espressione = espressione.Trim();

                    if (!forzaChiusura && espressione.Length > 0)
                        try
                        {
                            // Trasforma l'espressione in un array di byte e procede con l'invio
                            byte[] bufferEspr = Encoding.UTF8.GetBytes(espressione);
                            client.Send(bufferEspr);

                            // Riceve il risultato dell'espressione e lo converte in una stringa
                            int ricevuti = client.Receive(bufferRis);
                            string risultato = Encoding.UTF8.GetString(bufferRis, 0, ricevuti);

                            Console.WriteLine("Risultato = {0}", risultato);
                        }
                        catch (Exception e)
                        {
                            Console.Error.WriteLine(e.Message);
                        }
                }
                while (!forzaChiusura && espressione.Length > 0);

                // Al termine delle attività occorre chiudere il socket e rilasciare le risorse
                client.Shutdown(SocketShutdown.Both);
                client.Close();

                Console.WriteLine("Connessione chiusa.");
            }
            catch (Exception e)
            {
                Console.Error.WriteLine("Errore: {0}", e.Message);
            }
        }

        #region unmanaged

        [DllImport("Kernel32")]
        public static extern bool SetConsoleCtrlHandler(HandlerRoutine Handler, bool Add);

        public delegate bool HandlerRoutine(CtrlTypes CtrlType);

        public enum CtrlTypes
        {
            CTRL_C_EVENT = 0,
            CTRL_BREAK_EVENT,
            CTRL_CLOSE_EVENT,
            CTRL_LOGOFF_EVENT = 5,
            CTRL_SHUTDOWN_EVENT
        }

        #endregion

        private static bool ControllaEventiChiusura(CtrlTypes tipo)
        {
            switch (tipo)
            {
                case CtrlTypes.CTRL_C_EVENT:
                    forzaChiusura = true;
                    Console.WriteLine("CTRL+C");
                    break;

                case CtrlTypes.CTRL_BREAK_EVENT:
                    forzaChiusura = true;
                    Console.WriteLine("CTRL+BREAK");
                    break;

                case CtrlTypes.CTRL_CLOSE_EVENT:
                    forzaChiusura = true;
                    Console.WriteLine("Chiusura in corso...");
                    break;

                case CtrlTypes.CTRL_LOGOFF_EVENT:
                case CtrlTypes.CTRL_SHUTDOWN_EVENT:
                    forzaChiusura = true;
                    Console.WriteLine("Arresto in corso...");
                    break;
            }
            return true;
        }

    }

}
